from pprint import pprint
import os
import re
import ipaddress
from ..constants import PYVERSION, RE_IPV4_CIDR, RE_IPV4_PADDING, RE_FQDN,\
    RE_ASN, RE_EMAIL, RE_HASH, RE_URI_SCHEMES

from .ip import is_ipv4, is_ipv4_cidr, is_ipv6, is_ip

from .ztime import parse_timestamp

try:
    # py3
    from urllib.parse import urlparse
    import importlib
except ImportError:
    # py2
    from urlparse import urlparse


def ipv4_normalize(i):
    return RE_IPV4_PADDING.sub(r'\1\2', i)


def load_plugin(path, plugin):
    path = os.path.join(path, ('%s.py' % plugin))
    spec = importlib.util.spec_from_file_location(path, path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module


def is_valid(i):
    try:
        resolve_itype(i)
        return True
    except TypeError as e:
        return False


def is_hash(s):
    for h in RE_HASH:
        if re.match(RE_HASH[h], s):
            return h


def is_email(s):
    if re.match(RE_EMAIL, s):
        return True


def is_asn(s):
    if re.match(RE_ASN, s):
        return True


def is_fqdn(s):
    if RE_FQDN.match(s):
        return True


def is_url(s):
    u = urlparse(s)

    if not u:
        return

    if not re.match(RE_URI_SCHEMES, str(u.scheme)):
        return

    u = u.hostname

    if is_ip(u) or is_fqdn(u):
        return True

    if ':' in u:  # 192.168.1.1:81
        u1 = u.split(':')[0]
        if is_ipv4(u1) or is_fqdn(u1):
            return True


def is_url_broken(s):
    if PYVERSION == 2:
        s = s.encode('utf-8')

    u = urlparse('{}{}'.format('http://', s))

    if not re.match(RE_URI_SCHEMES, u.scheme):
        return

    if is_fqdn(u.hostname) or is_ipv4(u.hostname) or is_ipv6(u.hostname):
        return True


def resolve_itype(indicator, test_broken=False):
    itype = False
    if test_broken and is_url_broken(indicator):
        itype = 'broken_url'

    elif is_url(indicator):
        itype = 'url'

    elif is_hash(indicator):
        itype = is_hash(indicator)

    elif is_ipv4(indicator) or is_ipv4_cidr(indicator):
        itype = 'ipv4'

    elif is_ipv6(indicator):
        itype = 'ipv6'

    elif is_email(indicator):
        itype = 'email'

    elif is_fqdn(indicator):
        itype = 'fqdn'

    elif is_asn(indicator):
        itype = 'asn'

    if itype:
        return itype

    try:
        error = 'unknown itype for "{}"'.format(indicator)
    except UnicodeEncodeError:
        error = 'unknown itype for "{}"'.format(indicator.encode('utf-8'))

    raise TypeError(error)


def normalize_itype(i, itype=None):
    try:
        if resolve_itype(i['indicator']):
            return i
    except TypeError:
        pass

    i = _normalize_url(i)
    return i


def is_subdomain(i):
    itype = resolve_itype(i)
    if itype is not 'fqdn':
        return

    bits = i.split('.')
    if len(bits) > 2:
        bits.pop(0)
        return '.'.join(bits)


def is_ipv4_net(i):
    try:
        if resolve_itype(i) != 'ipv4':
            return False
    except TypeError:
        return False

    if not re.match(RE_IPV4_CIDR, i):
        return False

    if PYVERSION == 2:
        i = unicode(i)

    try:
        ipaddress.ip_network(i)
        return True
    except ValueError:
        return False


def _normalize_url(i):
    if resolve_itype(i['indicator'], test_broken=True) == 'broken_url':
        if PYVERSION == 2:
            i['indicator'] = i['indicator'].encode('utf-8')
        i['indicator'] = '{}{}'.format('http://', i['indicator'])

    return i


def url_to_fqdn(u):
    u = urlparse(u)
    return u.hostname


def list_to_csv(mylist):
    if len(mylist) > 0 and isinstance(mylist[0], dict):
        mylist = ''
    else:
        mylist = u','.join(mylist)

    return mylist



import ipaddress
import socket
import re

from csirtg_indicator.constants import RE_IPV4, RE_IPV4_CIDR


def is_valid_ip(i):
    try:
        ipaddress.ip_network(i)
    except ValueError as e:
        return False

    return True


def _test_socket(s, version=socket.AF_INET):
    try:
        socket.inet_pton(version, s)
        return True

    except socket.error:
        return None

    except UnicodeEncodeError:
        return False


def is_ip(i):
    return is_ipv4(i) or is_ipv6(i)


def is_ipv6(s):
    r = _test_socket(s, socket.AF_INET6)
    if r in [True, False]:
        return r

    try:
        # py2
        s = unicode(s)
    except UnicodeDecodeError:
        return False
    except NameError:
        pass

    try:
        ipaddress.IPv6Network(s)
        return True
    except ipaddress.AddressValueError:
        pass


def is_ipv4(s):
    r = _test_socket(s)
    if r in [True, False]:
        return r

    if re.match(RE_IPV4, s):
        return True


def is_ipv4_cidr(s):
    if not re.match(RE_IPV4_CIDR, s):
        return False

    try:
        # py2
        s = unicode(s)
    except UnicodeDecodeError:
        return False
    except NameError:
        pass

    try:
        ipaddress.ip_network(s)
        return True
    except ValueError as e:
        return False



from csirtg_indicator.utils.spamhaus import resolve_zen as resolve

from pprint import pprint
import arrow

CODES = {
    '127.0.0.2': {
        'tags': 'spam',
        'description': 'Direct UBE sources, spam operations & spam services',
    },
    '127.0.0.3': {
        'tags': 'spam',
        'description': 'Direct snowshoe spam sources detected via automation',
    },
    '127.0.0.4': {
        'tags': ['exploit', 'malware'],
        'description': 'CBL + customised NJABL. 3rd party exploits (proxies, '
                       'trojans, etc.)',
    },
    '127.0.0.5': {
        'tags': ['exploit', 'malware'],
        'description': 'CBL + customised NJABL. 3rd party exploits (proxies, '
                       'trojans, etc.)',
    },
    '127.0.0.6': {
        'tags': ['exploit', 'malware'],
        'description': 'CBL + customised NJABL. 3rd party exploits (proxies, '
                       'trojans, etc.)',
    },
    '127.0.0.7': {
        'tags': ['exploit', 'malware'],
        'description': 'CBL + customised NJABL. 3rd party exploits (proxies, '
                       'trojans, etc.)',
    },
    '127.0.0.9': {
        'tags': 'hijacked',
        'description': 'Spamhaus DROP/EDROP Data',
    },
}


def process(i, confidence=4, resolve_geo=False):
    if not i.is_ip():
        return

    r = resolve(i.indicator)
    r = CODES.get(str(r), None)

    if not r:
        return

    i2 = i.copy(**{
        'tags': r['tags'],
        'description': r['description'],
        'confidence': confidence,
        'provider': 'spamhaus.org',
        'reference': 'http://www.spamhaus.org/query/bl?ip=%s' % i.indicator,
        'reference_tlp': 'white',
        'last_at': arrow.utcnow(),
    })

    if resolve_geo:
        i2.geo_resolve()

    return i2


def main():
    from csirtg_indicator import Indicator
    i = Indicator('71.6.146.130')
    import logging
    logger = logging.getLogger('')
    logger.setLevel(logging.DEBUG)

    print(process(i))


if __name__ == '__main__':
    main()



from csirtg_indicator.utils.network import resolve_ns
import os

NAMESERVER = os.getenv('CSIRTG_INDICATOR_NAMESERVER', '1.1.1.1')


def _resolve(data, s):
    data = '%s.%s.spamhaus.org' % (data, s)
    data = resolve_ns(data, nameserver=NAMESERVER)
    if data and data[0]:
        return data[0]


def resolve_dbl(data):
    return _resolve(data, 'dbl')


def resolve_zen(data):
    data = '.'.join(reversed(data.split('.')))
    return _resolve(data, 'zen')



import socket
import dns.resolver
from dns.resolver import NoAnswer, NXDOMAIN, NoNameservers, Timeout
from dns.name import EmptyLabel
import re
import ipaddress

TIMEOUT = 5


try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse


def resolve_fqdn(host):
    if not host:
        return

    try:
        host = socket.gethostbyname(host)
        return host
    except Exception as e:
        return


def resolve_url(url):
    u = urlparse(url)
    return u.hostname


def resolve_ns(data, t='A', timeout=TIMEOUT, nameserver=None):
    resolver = dns.resolver.Resolver()
    resolver.timeout = timeout
    resolver.lifetime = timeout
    resolver.search = []

    if nameserver:
        resolver.nameservers = [nameserver]

    try:
        answers = resolver.query(data, t)

    except (NoAnswer, NXDOMAIN, EmptyLabel, NoNameservers, Timeout) as e:
        e = str(e)
        if e.startswith('The DNS operation timed out after'):
            return

        if 'The DNS response does not contain' in e or \
                'None of DNS query names exist' in e:
            return

        raise

    resp = []
    for rdata in answers:
        resp.append(rdata)

    return resp


def resolve_peers(indicator):
    if indicator.is_private() or not indicator.itype == 'ipv4':
        return indicator

    i = str(indicator.indicator)
    match = re.search('^(\S+)\/\d+$', i)
    if match:
        i = match.group(1)

    # cache it to the /24
    i = list(reversed(i.split('.')))
    i = '0.{}.{}.{}'.format(i[1], i[2], i[3])

    answers = resolve_ns('{}.{}'.format(i, 'peer.asn.cymru.com', timeout=15),
                         t='TXT')

    if answers is None or len(answers) == 0:
        return indicator

    if not indicator.peers:
        indicator.peers = []

    # Separate fields and order by netmask length
    # 23028 | 216.90.108.0/24 | US | arin | 1998-09-25
    # 701 1239 3549 3561 7132 | 216.90.108.0/24 | US | arin | 1998-09-25
    for p in answers:
        bits = str(p).replace('"', '').strip().split(' | ')
        asn = bits[0]
        prefix = bits[1]
        cc = bits[2]
        rir = bits[3]
        asns = asn.split(' ')
        for a in asns:
            indicator.peers.append({
                'asn': a,
                'prefix': prefix,
                'cc': cc,
                'rir': rir
            })

    return indicator


def is_valid_ip(i):
    try:
        ipaddress.ip_network(i)
        return True
    except ValueError as e:
        return False
